@@ -7,6 +7,7 @@ from .iedb import (
     IedbSMM_PMBEC,
     IedbNetMHCIIpan,
 )
+from .mixmhcpred import MixMHCpred
 from .mhcflurry import MHCflurry
 from .netchop import NetChop
 from .netmhc import NetMHC
@@ -16,10 +17,12 @@ from .netmhc_cons import NetMHCcons
 from .netmhc_pan import NetMHCpan
 from .netmhc_pan28 import NetMHCpan28
 from .netmhc_pan3 import NetMHCpan3
+from .netmhc_pan4 import NetMHCpan4, NetMHCpan4_BA, NetMHCpan4_EL
 from .netmhcii_pan import NetMHCIIpan
 from .random_predictor import RandomBindingPredictor
+from .unsupported_allele import UnsupportedAllele
 
-__version__ = "1.5.0"
+__version__ = "1.7.1"
 
 __all__ = [
     "BindingPrediction",
@@ -29,6 +32,7 @@ __all__ = [
     "IedbSMM",
     "IedbSMM_PMBEC",
     "IedbNetMHCIIpan",
+    "MixMHCpred",
     "MHCflurry",
     "NetChop",
     "NetMHC",
@@ -38,6 +42,10 @@ __all__ = [
     "NetMHCpan",
     "NetMHCpan28",
     "NetMHCpan3",
+    "NetMHCpan4",
+    "NetMHCpan4_BA",
+    "NetMHCpan4_EL",
     "NetMHCIIpan",
     "RandomBindingPredictor",
+    "UnsupportedAllele",
 ]
