@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017. Mount Sinai School of Medicine
+# Copyright (c) 2014-2019. Mount Sinai School of Medicine
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,10 +14,13 @@
 
 from __future__ import print_function, division, absolute_import
 
+import numpy as np
+
 from mhcnames import normalize_allele_name
 
 from .binding_prediction import BindingPrediction
 
+
 NETMHC_TOKENS = {
     "pos",
     "Pos",
@@ -29,6 +32,15 @@ NETMHC_TOKENS = {
     "Strong",
 }
 
+def check_stdout_error(stdout, program_name):
+    if "ERROR" in stdout.upper():
+        # if NetMHC* failed with an error then let's pull out the error
+        # message line and raise an exception with it
+        error_index = stdout.upper().index("ERROR")
+        stdout_after_error = stdout[error_index:]
+        error_line = stdout_after_error.split("\n")[0]
+        raise ValueError("%s failed - %s" % (program_name, error_line))
+
 def split_stdout_lines(stdout):
     """
     Given the standard output from NetMHC/NetMHCpan/NetMHCcons tools,
@@ -41,7 +53,9 @@ def split_stdout_lines(stdout):
     for l in stdout.split("\n"):
         l = l.strip()
         # wait for a line like '----------' before trying to parse entries
-        if l.startswith("-"):
+        # have to include multiple dashes here since NetMHC 4.0 sometimes
+        # gives negative positions in its "peptide" input mode
+        if l.startswith("---"):
             seen_dash = True
             continue
         if not seen_dash:
@@ -86,6 +100,24 @@ def clean_fields(fields, ignored_value_indices, transforms):
         cleaned_fields.append(cleaned_field)
     return cleaned_fields
 
+def valid_affinity(x):
+    """
+    Check that an IC50 affinity value is valid.
+
+    Parameters
+    ----------
+    x : float
+
+    Returns
+    -------
+    bool
+    """
+    if x is None:
+        return False
+    if np.isnan(x) or np.isinf(x):
+        return False
+    return x >= 0
+
 def parse_stdout(
         stdout,
         prediction_method_name,
@@ -94,9 +126,10 @@ def parse_stdout(
         offset_index,
         peptide_index,
         allele_index,
-        ic50_index,
-        rank_index,
-        log_ic50_index,
+        score_index,
+        rank_index=None,
+        ic50_index=None,
+
         ignored_value_indices={},
         transforms={}):
     """
@@ -105,8 +138,32 @@ def parse_stdout(
 
     Parameters
     ----------
+    stdout : str
+
+    prediction_method_name : str
+
+    key_index : int
+
+    offset_index : int
+
+    peptide_index : int
+
+    allele_index : int
+
+    score_index : int
+
+    rank_index : int
+
+    ic50_index : int
+
+    sequence_key_mapping : dict
+        Dictionary mapping sequence names (which might be hashes or truncated)
+        to the sequence names which should be used in the parsed
+        BindingPrediction objects
+
     ignored_value_indices : dict
-        Map from values to the positions we'll ignore them at. See clean_fields.
+        Map from values to the positions we'll ignore them at.
+        See clean_fields.
 
     transforms  : dict
         Map from field index to a transform function to be applied to values in
@@ -122,9 +179,21 @@ def parse_stdout(
         offset = int(fields[offset_index])
         peptide = str(fields[peptide_index])
         allele = str(fields[allele_index])
-        ic50 = float(fields[ic50_index])
-        rank = float(fields[rank_index]) if rank_index else 0.0
-        log_ic50 = float(fields[log_ic50_index])
+
+        if score_index is None:
+            score = None
+        else:
+            score = float(fields[score_index])
+
+        if rank_index is None:
+            rank = None
+        else:
+            rank = float(fields[rank_index])
+
+        if ic50_index is None:
+            ic50 = None
+        else:
+            ic50 = float(fields[ic50_index])
 
         key = str(fields[key_index])
         if sequence_key_mapping:
@@ -134,14 +203,20 @@ def parse_stdout(
             # identity function
             original_key = key
 
+        # if we have a bad IC50 score we might still get a salvageable
+        # log of the score. Strangely, this is necessary sometimes!
+        if ic50_index is not None and (not valid_affinity(ic50)) and np.isfinite(score):
+            # pylint: disable=invalid-unary-operand-type
+            ic50 = 50000 ** (1 - score)
+
         binding_predictions.append(BindingPrediction(
             source_sequence_name=original_key,
             offset=offset,
             peptide=peptide,
             allele=normalize_allele_name(allele),
+            score=score,
             affinity=ic50,
             percentile_rank=rank,
-            log_affinity=log_ic50,
             prediction_method_name=prediction_method_name))
     return binding_predictions
 
@@ -171,9 +246,9 @@ def parse_netmhc3_stdout(
         offset_index=0,
         peptide_index=1,
         allele_index=5,
+        score_index=2,
         ic50_index=3,
         rank_index=None,
-        log_ic50_index=2,
         ignored_value_indices={"WB": 4, "SB": 4})
 
 def parse_netmhc4_stdout(
@@ -208,7 +283,7 @@ def parse_netmhc4_stdout(
         allele_index=1,
         ic50_index=12,
         rank_index=13,
-        log_ic50_index=11)
+        score_index=11)
 
 def parse_netmhcpan28_stdout(
         stdout,
@@ -235,6 +310,7 @@ def parse_netmhcpan28_stdout(
      10  HLA-A*02:03    THIIIASSS   id0         0.040     32361.18   50.00
      11  HLA-A*02:03    HIIIASSSL   id0         0.515       189.74    4.00 <= WB
     """
+    check_stdout_error(stdout, "NetMHCpan-2.8")
     return parse_stdout(
         stdout=stdout,
         prediction_method_name=prediction_method_name,
@@ -245,7 +321,7 @@ def parse_netmhcpan28_stdout(
         allele_index=1,
         ic50_index=5,
         rank_index=6,
-        log_ic50_index=4)
+        score_index=4)
 
 def parse_netmhcpan3_stdout(
         stdout,
@@ -276,9 +352,55 @@ def parse_netmhcpan3_stdout(
         allele_index=1,
         ic50_index=12,
         rank_index=13,
-        log_ic50_index=11,
+        score_index=11,
         transforms=transforms)
 
+
+def parse_netmhcpan4_stdout(
+        stdout,
+        prediction_method_name="netmhcpan",
+        sequence_key_mapping=None,
+        mode="binding_affinity"):
+    """
+    # NetMHCpan version 4.0
+
+    # Tmpdir made /var/folders/jc/fyrvcrcs3sb8g4mkdg6nl_t80000gp/T//netMHCpanuH3SvY
+    # Input is in PEPTIDE format
+
+    # Make binding affinity predictions
+
+    HLA-A02:01 : Distance to training data  0.000 (using nearest neighbor HLA-A02:01)
+
+    # Rank Threshold for Strong binding peptides   0.500
+    # Rank Threshold for Weak binding peptides   2.000
+    -----------------------------------------------------------------------------------
+      Pos          HLA         Peptide       Core Of Gp Gl Ip Il        Icore        Identity     Score Aff(nM)   %Rank  BindLevel
+    -----------------------------------------------------------------------------------
+        1  HLA-A*02:01        SIINFEKL  SIINF-EKL  0  0  0  5  1     SIINFEKL         PEPLIST 0.1141340 14543.1 18.9860
+    -----------------------------------------------------------------------------------
+
+    Protein PEPLIST. Allele HLA-A*02:01. Number of high binders 0. Number of weak binders 0. Number of peptides 1
+    """
+
+    # the offset specified in "pos" (at index 0) is 1-based instead of 0-based. we adjust it to be
+    # 0-based, as in all the other netmhc predictors supported by this library.
+    transforms = {
+        0: lambda x: int(x) - 1,
+    }
+    return parse_stdout(
+        stdout=stdout,
+        prediction_method_name=prediction_method_name,
+        sequence_key_mapping=sequence_key_mapping,
+        key_index=10,
+        offset_index=0,
+        peptide_index=2,
+        allele_index=1,
+        score_index=11,
+        ic50_index=None if mode == "elution_score" else 12,
+        rank_index=12 if mode == "elution_score" else 13,
+        transforms=transforms)
+
+
 def parse_netmhccons_stdout(
         stdout,
         prediction_method_name="netmhccons",
@@ -314,7 +436,7 @@ def parse_netmhccons_stdout(
         allele_index=1,
         ic50_index=5,
         rank_index=6,
-        log_ic50_index=4)
+        score_index=4)
 
 def parse_netmhciipan_stdout(
         stdout,
@@ -343,6 +465,7 @@ def parse_netmhciipan_stdout(
          9         DRB1_0301       PKYVKQNTLKLAT    Sequence    2    YVKQNTLKL 0.575         0.442        418.70   6.00   9.999   <=WB
         10         DRB1_0301     ENPVVHFFKNIVTPR    Sequence    6    FFKNIVTPR 0.425         0.357       1049.04  32.00   9.999
     """
+    check_stdout_error(stdout, "NetMHCIIpan")
     return parse_stdout(
         stdout=stdout,
         prediction_method_name=prediction_method_name,
@@ -351,6 +474,6 @@ def parse_netmhciipan_stdout(
         offset_index=0,
         peptide_index=2,
         allele_index=1,
-        ic50_index=7,
-        rank_index=8,
-        log_ic50_index=6)
+        ic50_index=8,
+        rank_index=9,
+        score_index=7)
